@@ -4,8 +4,10 @@
 fit best estimate of magnetometer offsets using the algorithm from
 Bill Premerlani
 '''
+from __future__ import print_function
+from builtins import range
 
-import sys, time, os, math
+import sys
 
 # command line option handling
 from argparse import ArgumentParser
@@ -24,7 +26,7 @@ parser.add_argument("logs", metavar="LOG", nargs="+")
 args = parser.parse_args()
 
 from pymavlink import mavutil
-from pymavlink.rotmat import Vector3, Matrix3
+from pymavlink.rotmat import Vector3
 
 
 def noise():
@@ -89,7 +91,7 @@ def find_offsets(data, ofs):
         ofs = ofs - delta
 
         if args.verbose:
-            print ofs
+            print(ofs)
     return ofs
 
 
@@ -114,7 +116,7 @@ def magfit(logfile):
         if m.get_type() == "SENSOR_OFFSETS":
             # update offsets that were used during this flight
             offsets = Vector3(m.mag_ofs_x, m.mag_ofs_y, m.mag_ofs_z)
-        if m.get_type() == "RAW_IMU" and offsets != None:
+        if m.get_type() == "RAW_IMU" and offsets is not None:
             # extract one mag vector, removing the offsets that were
             # used during that flight to get the raw sensor values
             mag = Vector3(m.xmag, m.ymag, m.zmag) - offsets
